// logomg -> un filtro para virtualdub: logo con alfa y fadein-out
//
// Version 1.00 - 09-23-2005
// Version 1.01 - 09-26-2005 (initial state condition bug)
// Version 1.1  - 09-28-2005 (scripting support added)
//
// TODO -> optimizar funciones... operaciones binarias? MMX?

#include <windows.h>
#include <commctrl.h>
#include <commdlg.h>
#include <new>

#include "filter.h"
#include "vbitmap.h"
#include "resource.h"

#include "ScriptInterpreter.h"
#include "ScriptValue.h"
#include "ScriptError.h"

#include <stdio.h>
#include <math.h>


///////////////////////////////////////////////////////////////////////////
// DECLARACION de TIPOS de DATOS para FILTRO
///////////////////////////////////////////////////////////////////////////
typedef struct LogomgImage {
	PixDim    w;     //width
	PixDim    h;     //height
	Pixel32   *data; //pixels
} LogomgImage;

typedef struct LogomgFilterData {
	char	         szRutaLogo[MAX_PATH];
	char	         szRutaMasc[MAX_PATH];
	PixCoord         pos_x, pos_y;
	PixDim           max_x, max_y;
	Pixel32          opacidad,anterioropc;
	Pixel32          primero,lonin,duracion,lonout;
	bool	         bUsarAlfa;
	LogomgImage      logo,masc;
    IFilterPreview   *ifp;
} LogomgFilterData;


///////////////////////////////////////////////////////////////////////////
// FUNCIONES de APOYO
///////////////////////////////////////////////////////////////////////////
int VDRoundToInt(double x) {
	return (int)floor(x + 0.5);
}//VDRoundToInt()

void escaparCadena(char *dst, const char *org) {
	char *d = dst;

	for (int i=0;i<strlen(org);i++) {
		if (org[i]=='\\' || org[i]=='"') *d++ = '\\';
		*d++ = org[i];
	}
	*d = 0;
}//escaparCadena()

char * ExtractFileName(char *szFilePath) {
	char separadores[] = ":\\";
	char *l = szFilePath;
	char *p = strpbrk(szFilePath,separadores);
	while (p!=NULL) {
		l = p;
		p = strpbrk(p+1,separadores);
	}
	return l+1;
}//ExtractFileName()

BOOL LoadBMPFile(char *szFile, LogomgImage *bmpFile){
	BITMAPFILEHEADER bfh;
	BITMAPINFOHEADER bih;
	char *data;
	HANDLE f;
	DWORD leidos;
	//leemos los datos del archivo
	f = CreateFile(TEXT(szFile),GENERIC_READ,FILE_SHARE_READ,NULL,
				   OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,NULL);
	if (f == INVALID_HANDLE_VALUE) return FALSE;
	if (!ReadFile(f,&bfh,sizeof bfh,&leidos,NULL)) {
		CloseHandle(f);
		return FALSE;
	}
	if (!ReadFile(f,&bih,sizeof bih,&leidos,NULL)) {
		CloseHandle(f);
		return FALSE;
	}
	if (bih.biCompression!=BI_RGB){
		CloseHandle(f);
		MessageBox(0,"Bitmap Compressed","Error",0);
		return FALSE;
	}
	if (bih.biBitCount!=24){
		CloseHandle(f);
		MessageBox(0,"BitCount of Bitmap is not 24 Bit","Error",0);
		return FALSE;
	}
	SetFilePointer(f,bfh.bfOffBits,NULL,FILE_BEGIN);
	int tam = bih.biWidth*bih.biHeight*3;
	data = (char *)malloc(tam);
	if (!ReadFile(f,&data[0],tam,&leidos,NULL)) {
		free(data);
		CloseHandle(f);
		return FALSE;
	}
	CloseHandle(f);
	//convertimos de 24 a 32 bits
	bmpFile->w = bih.biWidth;
	bmpFile->h = bih.biHeight;
	bmpFile->data = (Pixel32 *)malloc(4*bmpFile->w*bmpFile->h);
	char *po,*pd;
	po=data;
	pd=(char *)bmpFile->data;
	int p=bih.biWidth*bih.biHeight;
	do {
		*pd++=*po++;      //R
		*pd++=*po++;      //G
		*pd++=*po++;      //B
        *pd++=(char)0xff; //A -> OPACO
	} while(--p);
	free(data);
	return TRUE;
}//LoadBMPFile()

void AplicarValorAlfa(const LogomgImage logo, Pixel32 alfa){
	Pixel32 *l=logo.data;
	alfa = alfa << 24;	
	int h = logo.h;
	do {
		int w = logo.w;
		do {
			*l = ((*l)&0x00ffffff) + (alfa);
			l++;
		} while(--w);
	} while(--h);
}//AplicarValorAlfa()

void AplicarMascaraAlfa(const LogomgImage logo, const LogomgImage masc, Pixel32 maxAlfa){
	Pixel32 *l=logo.data;
	Pixel32 *m=masc.data;
	Pixel32 alfa;
	maxAlfa = maxAlfa << 24;
	int h = logo.h;
	do {
		int w = logo.w;
		do {
			alfa = (*m) << 24;
			if (alfa>maxAlfa) alfa=maxAlfa;
			*l = ((*l)&0x00ffffff) + (alfa);
			l++;
			m++;
		} while(--w);
	} while(--h);
}//AplicarMascaraAlfa()

void VolcarImagen(VBitmap *destino, LogomgImage *logo, PixCoord x, PixCoord y) {
	//if (logo->w<=0 || logo->h<=0) return; 
	if (x+logo->w > destino->w) x=destino->w-logo->w;
	if (y+logo->h > destino->h) y=destino->h-logo->h;
	Pixel32 *dst,*src;
	PixOffset dstoff;
    src=logo->data;
	dst=destino->Address32(x,y+logo->h-1);
	dstoff = destino->pitch - 4*(logo->w);
	int h = logo->h;
	do {
		int w = logo->w;
		do {			
			*dst = *src;
			dst++;
			src++;
		} while(--w);
		dst = (Pixel32 *)((char *)dst + dstoff);
	} while(--h);	
} //VolcarImagen()

void VolcarImagenConAlfa(VBitmap *destino, LogomgImage *logo, PixCoord x, PixCoord y) {
	//if (logo->w<=0 || logo->h<=0) return;
	if (x+logo->w > destino->w) x=destino->w-logo->w;
	if (y+logo->h > destino->h) y=destino->h-logo->h;
	Pixel32 *dst,*src;
	PixOffset dstoff;
    src=logo->data;
	dst=destino->Address32(x,y+logo->h-1);
	dstoff = destino->pitch - 4*(logo->w);
	int h = logo->h;
	do {
		int w = logo->w;
		do {
			//aplicamos algoritmo de alfa-blending donde alfa -> [0..255]
			//result = (alpha*foreground + (255 - alpha)*background)/255
			//reordenado matemticamente: r=(a(f-b)+255b)/255;
			const Pixel32 ba = *dst++;
			const Pixel32 fo = *src++;
			const Pixel32 al = fo >> 24;
			Pixel32 rb = ((ba>>16)&0xff);
			Pixel32 gb = ((ba>> 8)&0xff);
			Pixel32 bb = ((ba    )&0xff);
			Pixel32 rf = ((fo>>16)&0xff);
			Pixel32 gf = ((fo>> 8)&0xff);
			Pixel32 bf = ((fo    )&0xff);
			rb = VDRoundToInt((al*(rf-rb)+(255*rb))/255) << 16;
			gb = VDRoundToInt((al*(gf-gb)+(255*gb))/255) << 8;
			bb = VDRoundToInt((al*(bf-bb)+(255*bb))/255);
			dst[-1] = rb + gb + bb;
		} while(--w);
		dst = (Pixel32 *)((char *)dst + dstoff);
	} while(--h);	
} //VolcarImagenConAlfa()

const char *SeleccionarImagen(HWND hwnd, const char *oldfn) {
	OPENFILENAME ofn;
	static char szFile[MAX_PATH];
	char szFileTitle[MAX_PATH];
	if (oldfn) strcpy(szFile, oldfn);
	szFileTitle[0]=0;
	ofn.lStructSize			= sizeof(OPENFILENAME); //OPENFILENAME_SIZE_VERSION_400;
	ofn.hwndOwner			= hwnd;
	ofn.lpstrFilter			= "Bitmap file (*.bmp)\0*.bmp\0All files (*.*)\0*.*\0";
	ofn.lpstrCustomFilter	= NULL;
	ofn.nFilterIndex		= 1;
	ofn.lpstrFile			= szFile;
	ofn.nMaxFile			= sizeof szFile;
	ofn.lpstrFileTitle		= szFileTitle;
	ofn.nMaxFileTitle		= sizeof szFileTitle;
	ofn.lpstrInitialDir		= NULL;
	ofn.lpstrTitle			= "Select image";
	ofn.Flags				= OFN_EXPLORER | OFN_PATHMUSTEXIST | OFN_FILEMUSTEXIST | OFN_HIDEREADONLY | OFN_ENABLESIZING;
	ofn.lpstrDefExt			= NULL;
	if (GetOpenFileName(&ofn)) return szFile;
	return NULL;
}//SeleccionaImagen()

void ActualizarParametrosFiltro(HWND hDlg, LogomgFilterData *mfd) {
	long pos_x,pos_y,primero,lonin,duracion,lonout;
	BOOL success;
	pos_x = GetDlgItemInt(hDlg, IDC_XPOS, &success, FALSE);
	if (!success) {
		SetFocus(GetDlgItem(hDlg, IDC_XPOS));
		MessageBeep(MB_ICONEXCLAMATION);
		return;
	}
	pos_y = GetDlgItemInt(hDlg, IDC_YPOS, &success, FALSE);
	if (!success) {
		SetFocus(GetDlgItem(hDlg, IDC_YPOS));
		MessageBeep(MB_ICONEXCLAMATION);
		return;
	}
	primero = GetDlgItemInt(hDlg, IDC_START, &success, FALSE);
	if (!success) {
		SetFocus(GetDlgItem(hDlg, IDC_START));
		MessageBeep(MB_ICONEXCLAMATION);
		return;
	}
	lonin = GetDlgItemInt(hDlg, IDC_FADEINLEN, &success, FALSE);
	if (!success) {
		SetFocus(GetDlgItem(hDlg, IDC_FADEINLEN));
		MessageBeep(MB_ICONEXCLAMATION);
		return;
	}
	duracion = GetDlgItemInt(hDlg, IDC_DURATION, &success, FALSE);
	if (!success) {
		SetFocus(GetDlgItem(hDlg, IDC_DURATION));
		MessageBeep(MB_ICONEXCLAMATION);
		return;
	}
	lonout = GetDlgItemInt(hDlg, IDC_FADEOUTLEN, &success, FALSE);
	if (!success) {
		SetFocus(GetDlgItem(hDlg, IDC_FADEOUTLEN));
		MessageBeep(MB_ICONEXCLAMATION);
		return;
	}
	if (pos_x != mfd->pos_x ||
		pos_y != mfd->pos_y ||
		primero != mfd->primero ||
		lonin != mfd->lonin ||
		duracion != mfd->duracion ||
		lonout != mfd->lonout) 
	{
		mfd->pos_x = pos_x;
		mfd->pos_y = pos_y;
		mfd->primero = primero;
		mfd->lonin = lonin;
		mfd->duracion = duracion;
		mfd->lonout = lonout;
		mfd->ifp->RedoFrame();
	}
}//ActualizarParametrosFiltro()

void CambiarEstadoControles(HWND hdlg) {
	char cad[2];
	bool botros = 0!=GetDlgItemText(hdlg, IDC_LOGOFILE, &cad[0], sizeof cad);
	bool balfa  = 0!=IsDlgButtonChecked(hdlg, IDC_USEALPHA);
	EnableWindow(GetDlgItem(hdlg,IDC_USEALPHA),botros);
	EnableWindow(GetDlgItem(hdlg,IDC_ALPHAFILE),botros & balfa );
	EnableWindow(GetDlgItem(hdlg,IDC_ALPHAFILE_BROWSE),botros & balfa);
	EnableWindow(GetDlgItem(hdlg,IDC_XPOS),botros);
	EnableWindow(GetDlgItem(hdlg,IDC_SPIN_XPOS),botros);
	EnableWindow(GetDlgItem(hdlg,IDC_YPOS),botros);
	EnableWindow(GetDlgItem(hdlg,IDC_SPIN_YPOS),botros);
	EnableWindow(GetDlgItem(hdlg,IDC_OPACITY),botros);
	EnableWindow(GetDlgItem(hdlg,IDC_STATIC_OPACITY),botros);
	EnableWindow(GetDlgItem(hdlg,IDC_START),botros);
	EnableWindow(GetDlgItem(hdlg,IDC_FADEINLEN),botros);
	EnableWindow(GetDlgItem(hdlg,IDC_DURATION),botros);
	EnableWindow(GetDlgItem(hdlg,IDC_FADEOUTLEN),botros);
}//CambiarEstadoControles()

void LogoUpdateOpacity(HWND hDlg, LogomgFilterData *mfd) {
	int opacidad = SendDlgItemMessage(hDlg,IDC_OPACITY,TBM_GETPOS,0,0);
	if (opacidad != mfd->opacidad) {
		mfd->opacidad = opacidad;
		mfd->ifp->UndoSystem();
		mfd->ifp->RedoSystem();
		SetDlgItemInt(hDlg, IDC_STATIC_OPACITY,mfd->opacidad,false);
	}
} //LogoUpdateOpacity()


BOOL CALLBACK logomgDlgProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam) {
	LogomgFilterData *mfd = (struct LogomgFilterData *)GetWindowLong(hDlg, DWL_USER);
	switch (message) {
        case WM_INITDIALOG: {
				mfd = (LogomgFilterData *)lParam;
				SetWindowLong(hDlg, DWL_USER, (LONG)mfd);

				SetDlgItemText(hDlg, IDC_LOGOFILE, mfd->szRutaLogo);
				CheckDlgButton(hDlg, IDC_USEALPHA, mfd->bUsarAlfa);
				SetDlgItemText(hDlg, IDC_ALPHAFILE, mfd->szRutaMasc);
				SetDlgItemInt(hDlg, IDC_XPOS, mfd->pos_x, TRUE);
				SetDlgItemInt(hDlg, IDC_YPOS, mfd->pos_y, TRUE);
				SendDlgItemMessage(hDlg, IDC_SPIN_XPOS, UDM_SETRANGE, 0, MAKELONG(mfd->max_x,0));
				SendDlgItemMessage(hDlg, IDC_SPIN_YPOS, UDM_SETRANGE, 0, MAKELONG(0,mfd->max_y));
				SendDlgItemMessage(hDlg, IDC_OPACITY, TBM_SETRANGE, TRUE, MAKELONG(0,255));
				SendDlgItemMessage(hDlg, IDC_OPACITY, TBM_SETPOS, TRUE, mfd->opacidad);
				SetDlgItemInt(hDlg, IDC_STATIC_OPACITY,mfd->opacidad,false);
				SetDlgItemInt(hDlg, IDC_START, mfd->primero, TRUE);
				SetDlgItemInt(hDlg, IDC_FADEINLEN, mfd->lonin, TRUE);
				SetDlgItemInt(hDlg, IDC_DURATION, mfd->duracion, TRUE);
				SetDlgItemInt(hDlg, IDC_FADEOUTLEN, mfd->lonout, TRUE);
				mfd->ifp->InitButton(GetDlgItem(hDlg, IDC_PREVIEW));
				CambiarEstadoControles(hDlg);
			}
            return TRUE;

        case WM_COMMAND:                      
			switch(LOWORD(wParam)) {
			case IDOK:
				mfd->ifp->Close();
				EndDialog(hDlg, 0);
				return TRUE;

			case IDCANCEL:
				mfd->ifp->Close();
                EndDialog(hDlg, 1);
                return TRUE;

			case IDC_USEALPHA:
				mfd->bUsarAlfa = 0 != SendMessage((HWND)lParam, BM_GETCHECK, 0, 0);
				mfd->ifp->UndoSystem();
				mfd->ifp->RedoSystem();
				CambiarEstadoControles(hDlg);
				return TRUE;

			case IDC_LOGOFILE:
				if (HIWORD(wParam) == EN_KILLFOCUS) {
					GetDlgItemText(hDlg, IDC_LOGOFILE, mfd->szRutaLogo, sizeof mfd->szRutaLogo);
                    CambiarEstadoControles(hDlg);
					mfd->ifp->UndoSystem();
					mfd->ifp->RedoSystem();
				}
				return TRUE;

			case IDC_ALPHAFILE:
				if (HIWORD(wParam) == EN_KILLFOCUS) {
					GetDlgItemText(hDlg, IDC_ALPHAFILE, mfd->szRutaMasc, sizeof mfd->szRutaMasc);
					mfd->ifp->UndoSystem();
					mfd->ifp->RedoSystem();
				}
				return TRUE;

			case IDC_LOGOFILE_BROWSE:
				if (const char *fn = SeleccionarImagen(hDlg, mfd->szRutaLogo)) {
					SetDlgItemText(hDlg, IDC_LOGOFILE, fn);
					strcpy(mfd->szRutaLogo, fn);
					CambiarEstadoControles(hDlg);
					mfd->ifp->UndoSystem();
					mfd->ifp->RedoSystem();
				}
				return TRUE;

			case IDC_ALPHAFILE_BROWSE:
				if (const char *fn = SeleccionarImagen(hDlg, mfd->szRutaMasc)) {
					SetDlgItemText(hDlg, IDC_ALPHAFILE, fn);
					strcpy(mfd->szRutaMasc, fn);
					mfd->ifp->UndoSystem();
					mfd->ifp->RedoSystem();
				}
				return TRUE;

			case IDC_XPOS:
			case IDC_YPOS:
			case IDC_START:
			case IDC_FADEINLEN:
			case IDC_DURATION:
			case IDC_FADEOUTLEN:
				if (HIWORD(wParam) == EN_KILLFOCUS)	ActualizarParametrosFiltro(hDlg, mfd);
				return TRUE;

			case IDC_PREVIEW:
				mfd->ifp->Toggle(hDlg);
				return TRUE;

            }
            break;

		case WM_HSCROLL:
		case WM_VSCROLL:
			if (lParam)
				switch(GetWindowLong((HWND)lParam, GWL_ID)) {
				case IDC_OPACITY:
					LogoUpdateOpacity(hDlg, mfd);
					break;
				case IDC_XPOS:
				case IDC_YPOS:
				case IDC_SPIN_XPOS:
				case IDC_SPIN_YPOS:
					ActualizarParametrosFiltro(hDlg, mfd);
					break;
				
				}
			return TRUE;
    }
    return FALSE;
}//logomgDlgProc()


///////////////////////////////////////////////////////////////////////////
// FUNCIONES del FILTRO
///////////////////////////////////////////////////////////////////////////
int logomg_init(FilterActivation *fa, const FilterFunctions *ff) {
	LogomgFilterData *mfd = (LogomgFilterData *)fa->filter_data;
	mfd->szRutaLogo[0] = 0;
	mfd->szRutaMasc[0] = 0;
	mfd->pos_x = 630;
	mfd->pos_y = 485;
	mfd->bUsarAlfa = false;
	mfd->opacidad = 200;
	mfd->anterioropc = 255;//bmp se carga opaco
	mfd->primero = 25;     //fotogramas
	mfd->lonin = 25*2;     //fotogramas 
	mfd->duracion = 25*5;  //fotogramas
	mfd->lonout = 25*2;    //fotogramas
	//paranoia?
	mfd->logo.w=mfd->logo.h = 0;
    mfd->masc.w=mfd->masc.h = 0;
	mfd->logo.data = NULL;
	mfd->masc.data = NULL;
	return 0;
}//logomg_init()

long logomg_param(FilterActivation *fa, const FilterFunctions *ff) {
	fa->dst.offset = fa->src.offset;
	LogomgFilterData *mfd = (LogomgFilterData *)fa->filter_data;
	mfd->max_x = fa->dst.w;
	mfd->max_y = fa->dst.h;
	return 0;
}//logomg_param()

int logomg_start(FilterActivation *fa, const FilterFunctions *ff) {
	LogomgFilterData *mfd = (LogomgFilterData *)fa->filter_data;
	if (!LoadBMPFile(mfd->szRutaLogo,&mfd->logo)) {
		ff->Except("Couldn't load logo bmp file.");
		return -1;
	}
    if (mfd->bUsarAlfa) 
	if (!LoadBMPFile(mfd->szRutaMasc,&mfd->masc)) {
		ff->Except("Couldn't load mask bmp file.");
		return -1;
	} else if (mfd->logo.w!=mfd->masc.w || mfd->logo.h!=mfd->masc.h) {
		ff->Except("Mask bmp file has different dimensions from logo bmp file.");
		return -1;
	} else {
		AplicarMascaraAlfa(mfd->logo,mfd->masc,mfd->anterioropc);
	}
	return 0;
}//logomg_start()

int logomg_stop(FilterActivation *fa, const FilterFunctions *ff) {
	LogomgFilterData *mfd = (LogomgFilterData *)fa->filter_data;
    if (mfd->logo.data != NULL) free(mfd->logo.data);
	mfd->logo.data = NULL;
    if (mfd->masc.data != NULL) free(mfd->masc.data);
	mfd->masc.data = NULL;
	return 0;
}//logomg_stop()

int logomg_config(FilterActivation *fa, const FilterFunctions *ff, HWND hWnd) {
	LogomgFilterData *mfd = (LogomgFilterData *)fa->filter_data;
	LogomgFilterData *mfdbak = mfd;
	mfd->ifp = fa->ifp;
	int ret=DialogBoxParam(fa->filter->module->hInstModule, MAKEINTRESOURCE(IDD_FILTER_LOGO), hWnd, logomgDlgProc, (LONG)mfd);
	if (!ret) mfd=mfdbak;
	return ret;
}//logomg_config()

int logomg_run(const FilterActivation *fa, const FilterFunctions *ff) {
	LogomgFilterData *mfd = (LogomgFilterData *)fa->filter_data;
	long f = fa->pfsi->lCurrentSourceFrame;
	//establecemos puntos lmite
	unsigned long p,i,d,o;
	p=mfd->primero;      // (p)rincipio (punto 0)
	i=mfd->lonin;        // lmite fade-(i)n
	d=i + mfd->duracion; // lmite (d)uracion
	o=d + mfd->lonout;   // lmite fade-(o)ut
	//comprobamos que est en la ventana de proceso
	if (f>=p && f<=p+o) {
		Pixel8 alfa = 0x00;
		f -= p; //fotograma actual relativo a nuestro punto 0
		//buscamos zona y calculamos alfa
        if (f<i) {
			// fade-in
			alfa=VDRoundToInt(mfd->opacidad*f/mfd->lonin);			
		} else if (f<d) {
			// full-visible
			alfa=mfd->opacidad;
		} else if (f<=o) {
			// fade-out
			alfa=mfd->opacidad-VDRoundToInt(mfd->opacidad*(f-d)/mfd->lonout);
		} else {
			// not visible
			alfa=0x00;
		}
		//actuamos
		if (alfa!=mfd->anterioropc) {
			mfd->anterioropc = alfa;
			if (mfd->bUsarAlfa) AplicarMascaraAlfa(mfd->logo,mfd->masc,alfa);
			else AplicarValorAlfa(mfd->logo,alfa);
		}
		if (mfd->bUsarAlfa || mfd->opacidad<255) VolcarImagenConAlfa(&fa->dst, &mfd->logo, mfd->pos_x, mfd->pos_y);
        else VolcarImagen(&fa->dst, &mfd->logo, mfd->pos_x, mfd->pos_y);
	}
	return 0;
}//logomg_run()

void logomg_string(const FilterActivation *fa, const FilterFunctions *ff, char *buf) {
	LogomgFilterData *mfd = (LogomgFilterData *)fa->filter_data;

	sprintf(buf," (x:%u y:%u o:%u s:%u fi:%u d:%u fo:%u \"%s\")",
		    mfd->pos_x,
	 	    mfd->pos_y,
		    mfd->opacidad,
		    mfd->primero,
		    mfd->lonin,
		    mfd->duracion,
		    mfd->lonout,
			ExtractFileName(mfd->szRutaLogo) );
}//logomg_string()


///////////////////////////////////////////////////////////////////////////
// FUNCIONES de SCRIPTING
///////////////////////////////////////////////////////////////////////////
void logomg_script_config(IScriptInterpreter *isi, void *lpVoid, CScriptValue *argv, int argc) {
	FilterActivation *fa  = (FilterActivation *)lpVoid;
	LogomgFilterData *mfd = (LogomgFilterData *)fa->filter_data;

	mfd->pos_x    = argv[0].asInt();
	mfd->pos_y    = argv[1].asInt();
	mfd->opacidad = argv[2].asInt();
	mfd->primero  = argv[3].asInt();
	mfd->lonin    = argv[4].asInt();
	mfd->duracion = argv[5].asInt();
	mfd->lonout   = argv[6].asInt();

	mfd->szRutaLogo[0] = 0;
	strncpy(mfd->szRutaLogo, *argv[7].asString(), sizeof mfd->szRutaLogo);
	mfd->szRutaLogo[sizeof mfd->szRutaLogo - 1] = 0;

	mfd->bUsarAlfa = argv[8].asInt() && (argc>9);
	if (mfd->bUsarAlfa) {
		mfd->szRutaMasc[0] = 0;
		strncpy(mfd->szRutaMasc, *argv[9].asString(), sizeof mfd->szRutaMasc);
		mfd->szRutaMasc[sizeof mfd->szRutaMasc - 1] = 0;
	}
}//logomg_script_config()

ScriptFunctionDef logomg_func_defs[]={
	{ (ScriptFunctionPtr)logomg_script_config, "Config", "0iiiiiiisis" },
	{ (ScriptFunctionPtr)logomg_script_config, NULL, "0iiiiiiisi" },
	{ NULL },
};

CScriptObject logomg_obj={
	NULL, logomg_func_defs
};

bool logomg_script_line(FilterActivation *fa, const FilterFunctions *ff, char *buf, int buflen) {
	LogomgFilterData *mfd = (LogomgFilterData *)fa->filter_data;
	char logo[1024],masc[1024];
    if ((mfd->bUsarAlfa) && (strlen(mfd->szRutaMasc)>0)) {
		escaparCadena(logo,mfd->szRutaLogo);
		escaparCadena(masc,mfd->szRutaMasc);
		_snprintf(buf,buflen,"Config(%u,%u,%u,%u,%u,%u,%u,\"%s\",%u,\"%s\")",
			      mfd->pos_x, mfd->pos_y, mfd->opacidad, 
				  mfd->primero, mfd->lonin, mfd->duracion, mfd->lonout,
				  logo, mfd->bUsarAlfa, masc);
	} else {
		escaparCadena(logo,mfd->szRutaLogo);
		_snprintf(buf,buflen,"Config(%u,%u,%u,%u,%u,%u,%u,\"%s\",%u)",
			      mfd->pos_x, mfd->pos_y, mfd->opacidad, 
				  mfd->primero, mfd->lonin, mfd->duracion, mfd->lonout,
				  logo, mfd->bUsarAlfa);
	}
	return true;
}//logomg_script_line()


///////////////////////////////////////////////////////////////////////////
// DEFINICION del FILTRO
///////////////////////////////////////////////////////////////////////////
FilterDefinition filterDef_logomg={
	0,0,NULL,                           // next, prev, module
	"Logomg (1.1)",                     // name 
	"Logo with alpha mask and fading",  // desc
	"Miguel Gesteiro",                  // maker
	NULL,                               // private_data
	sizeof(LogomgFilterData),           // inst_data_size
	logomg_init,                        // initProc
	NULL,                               // deinitProc
	logomg_run,                         // runProc
	logomg_param,                       // paramProc
	logomg_config,                      // configProc
	logomg_string,                      // stringProc
	logomg_start,                       // startProc
	logomg_stop,                        // endProc
	&logomg_obj,                        // script_obj
	logomg_script_line                  // fssProc
};//filterDef_logomg{}


///////////////////////////////////////////////////////////////////////////
// INICIALIZACION del FILTRO
///////////////////////////////////////////////////////////////////////////
extern "C" int __declspec(dllexport)  __cdecl VirtualdubFilterModuleInit2(FilterModule *fm, const FilterFunctions *ff, int& vdfd_ver, int& vdfd_compat);
extern "C" void __declspec(dllexport) __cdecl VirtualdubFilterModuleDeinit(FilterModule *fm, const FilterFunctions *ff);

static FilterDefinition *fd_logomg;

///////////////////////////////////////////////////////////////////////////

int __declspec(dllexport) __cdecl VirtualdubFilterModuleInit2(FilterModule *fm, const FilterFunctions *ff, int& vdfd_ver, int& vdfd_compat) {
    if (!(fd_logomg = ff->addFilter(fm, &filterDef_logomg, sizeof(FilterDefinition))))
        return 1;
	vdfd_ver    = VIRTUALDUB_FILTERDEF_VERSION;
	vdfd_compat = VIRTUALDUB_FILTERDEF_COMPATIBLE;
    return 0;
}

void __declspec(dllexport) __cdecl VirtualdubFilterModuleDeinit(FilterModule *fm, const FilterFunctions *ff) {
    ff->removeFilter(fd_logomg);
}

///////////////////////////////////////////////////////////////////////////